#!/usr/bin/ruby
%w(quran pry paint).each do |lib|
  require lib
rescue LoadError
  if lib == 'quran'
    warn "The '#{lib}' library code could not be required (#{$!.message})\n" \
         "If running from a copy of Quran.rb, run: ruby -Ilib bin/quran\n"
  else
    warn "The '#{lib}' library code could not be required (#{$!.message})\n" \
         "If running from a copy of Quran.rb, double check you have run: \n" \
         "gem install -g gem.deps.rb"
  end
  exit 1
end

# Set the process title
Process.setproctitle 'quran.rb'
sleep 0.4

# Verify the Qur'an repository is readable
quran_path = File.expand_path ENV['QURAN_PATH'] || '~/Quran'
if not File.readable?(quran_path)
  warn "A readable copy of the Qur'an was not found at #{quran_path}\n." \
       "Clone a copy from https://github.com/0x1eef/Quran.git"
  exit 1
end

# Print a welcome message before session starts
highlight_class = lambda { Paint[_1, :bold] }
quran_title     = Paint['Quran.rb', :red, :bold]
start_message   = <<-WELCOME
#{Paint['Welcome', :bold]}

Welcome to what is intended to be a Ruby developer's environment
for reading and learning about #{Paint[%q<The Qur'an>, :green, :bold]}, and learning the interface
of the Ruby library #{quran_title}. To get started, enter 'quran' into
the command prompt.

'ls quran' will show you what methods an instance of the #{highlight_class.call 'Quran'}
class implements. For example, 'quran.surahs' returns an array of
#{highlight_class.call 'Quran::Surah'} objects while 'quran.ayahs' returns an array of
#{highlight_class.call 'Quran::Ayah'} objects.

WELCOME
Pry.hooks.add_hook :before_session, :start do |out, b, pry|
  pry.run_command "clear-screen"
  out.puts(start_message)
end

# Start Pry
Pry.start TOPLEVEL_BINDING, prompt_name: "#{quran_title} ",
                            extra_sticky_locals: {quran: Quran.new(quran_path)}
