class Quran
  require "ostruct"
  require "yaml"
  require_relative "quran/surah"
  require_relative "quran/ayah"
  require_relative "quran/cursor"
  require_relative "quran/title"

  attr_reader :path

  ##
  # @param [String] path
  #  The path to The Qur'an.
  def initialize(path)
    @path = File.expand_path(path)
  end

  ##
  # @return [Array<Quran::Surah>]
  #  Returns an array of {Quran::Surah} objects.
  def surahs
    Dir[File.join(path, "*")].map do |path|
      desc_path = File.join(path, "DESCRIPTION.YML")
      next unless File.readable?(desc_path)

      desc = OpenStruct.new YAML.load_file(desc_path)
      Surah.new(
        Title.new(desc.title),
        desc.description,
        desc.index,
        path
      )
    end.compact.sort_by(&:index)
  end

  ##
  # @return [Array<Quran::Ayah]
  #  Returns all ayahs from The Qur'an as an array of
  #  {Quran::Ayah} objects.
  def ayahs
    surahs.flat_map(&:ayahs)
  end
end
