class Quran::Ayah < Struct.new(:index, :paths, :surah)
  require "opener"

  def initialize(index, paths, surah)
    super
    return unless multiple_parts?
    self.paths = paths.sort_by(&method(:part_index))
  end

  ##
  # Opens an Ayah as an image with the default application for
  # viewing images.
  #
  # @param [Integer] part
  #  In case of an Ayah broken up into parts, this argument chooses
  #  what part to pick.
  #
  # @return [Integer]
  #  Returns the PID of the spawned process.
  def open(part = 1)
    Opener.spawn paths[part - 1]
  end

  ##
  # Returns true when "self" and "ayah" are the same ayah.
  #
  # @param [Quran::Ayah] ayah
  #  An ayah.
  #
  # @return [Boolean]
  def eql?(ayah)
    return false unless self.class === ayah
    index == ayah.index && surah.index == ayah.surah.index
  end
  alias_method :==, :eql?

  def multiple_parts?
    paths.size > 1
  end

  private

  def part_index(path)
    basename = File.basename(path, File.extname(path))
    Integer basename[-1]
  end
end
